@@ -88,15 +88,17 @@ bool QSvgIOHandlerPrivate::load(QIODevice *device)
         const QByteArray &ba = buf->data();
         res = r.load(QByteArray::fromRawData(ba.constData() + buf->pos(), ba.size() - buf->pos()));
         buf->seek(ba.size());
+#ifndef QT_NO_COMPRESS
     } else if (q->format() == "svgz") {
         res = r.load(device->readAll());
+#endif
     } else {
         xmlReader.setDevice(device);
         res = r.load(&xmlReader);
     }
 
     if (res) {
-        defaultSize = QSize(r.viewBox().width(), r.viewBox().height());
+        defaultSize = r.defaultSize();
         loaded = true;
     }
 
@@ -124,24 +126,20 @@ bool QSvgIOHandler::canRead() const
     if (d->loaded && !d->readDone)
         return true;        // Will happen if we have been asked for the size
 
-    QByteArray buf = device()->peek(8);
+    QByteArray buf = device()->peek(16);
+#ifndef QT_NO_COMPRESS
     if (buf.startsWith("\x1f\x8b")) {
         setFormat("svgz");
         return true;
-    } else if (buf.contains("<?xml") || buf.contains("<svg") || buf.contains("<!--")) {
+    } else
+#endif
+    if (buf.contains("<?xml") || buf.contains("<svg") || buf.contains("<!--") || buf.contains("<!DOCTYPE svg")) {
         setFormat("svg");
         return true;
     }
     return false;
 }
 
-
-QByteArray QSvgIOHandler::name() const
-{
-    return "svg";
-}
-
-
 bool QSvgIOHandler::read(QImage *image)
 {
     if (!d->readDone && d->load(device())) {
@@ -171,7 +169,13 @@ bool QSvgIOHandler::read(QImage *image)
             t.translate(tr1.x(), tr1.y());
             bounds = t.mapRect(bounds);
         }
-        *image = QImage(finalSize, QImage::Format_ARGB32_Premultiplied);
+        if (image->size() != finalSize || !image->reinterpretAsFormat(QImage::Format_ARGB32_Premultiplied)) {
+            *image = QImage(finalSize, QImage::Format_ARGB32_Premultiplied);
+            if (!finalSize.isEmpty() && image->isNull()) {
+                qWarning("QSvgIOHandler: QImage allocation failed (size %i x %i)", finalSize.width(), finalSize.height());
+                return false;
+            }
+        }
         if (!finalSize.isEmpty()) {
             image->fill(d->backColor.rgba());
             QPainter p(image);
@@ -256,8 +260,12 @@ bool QSvgIOHandler::supportsOption(ImageOption option) const
 
 bool QSvgIOHandler::canRead(QIODevice *device)
 {
-    QByteArray buf = device->peek(8);
-    return buf.startsWith("\x1f\x8b") || buf.contains("<?xml") || buf.contains("<svg") || buf.contains("<!--");
+    QByteArray buf = device->peek(16);
+    return
+#ifndef QT_NO_COMPRESS
+            buf.startsWith("\x1f\x8b") ||
+#endif
+            buf.contains("<?xml") || buf.contains("<svg") || buf.contains("<!--") || buf.contains("<!DOCTYPE svg");
 }
 
 QT_END_NAMESPACE
